/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** stats.h - main header file
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: stats.h,v 1.37 2002/04/07 17:53:52 wcampbel Exp $
*/

#ifndef STATS_H
#define STATS_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/time.h>
#include <errno.h>
#include <ctype.h>
#include <sys/stat.h>
#include "config.h"
#include "dlink.h"

/* Standard settings... ONLY change these if you know what
** you are doing.  The service may fail to work if not
**
** NOTE:  These are used to get rid of "magic numbers" in the code
**
** NOTE:  NEVER Change BUFSIZE
*/
#define BUFSIZE			512
#define CONFIG_NAME		"etc/stats.cfg"
#define TLD_FILE		"data/tlds.nfo"
#define STATS_DB		"data/stats.db"
#define NSTATS_DB		"data/nstats.db"
#define PIDFILE			"etc/stats.pid"
#define LOGFILE			"logs/stats.log"
#define SPLITLOG		"logs/split.log"
#define USERFILE		"etc/users.lst"
#define BACKUPLOGFMT		"logs/stats-%m-%d.log"
#define BACKUPSPLOGFMT		"logs/split-%m-%d.log"
#define TIMEFMT			"%H:%M[%m/%d/%Y]"
#define FTMFMT                  "%A, %Ef %Y, %I:%M%p %Z"
/* L10n -Hwy */
/* #define FTMFMT			"%A, %d %b %Y, %I:%M%p %Z" */
#define IRCTFMT                 "%A %EF %Y -- %H:%M:%S %z"
/* L10n -Hwy */
/* #define IRCTFMT			"%A %B %d %Y -- %H:%M:%S " */
#define MAXHOST			63
#define MAXPASS			32
#define MAXNICK			32
#define MAXUSER			10
#define MAXTLD			10 /* Maximum length of a TLD, used to size
				   ** the TLD struct
				   */
#define MAXPARA			15 /* Maximum number of parameters */
#define LINE			80 /* Don't change this */
#define TIMEBUF			100 /* Accepts time formatting */
#define SPAMLEN			8  /* Maximum length of the Spam service's
				   ** nick name
				   */
#define MAXCOLLISIONS           20 /* A tunable, the maximum number of
				   ** nick collides permitted in COLLIDETIME
				   ** before "committing suicide"
				   */
#define SPAMCHANGETIME		300 /* The time between the Spam service's
				    ** nick changes
				    */
#define STATSSAVETIME		120 /* The time between db syncs */
#define COLLIDETIME		60  /* see MAXCOLLISIONS */
#define PINGERTIME		60  /* Time between sending out PING's */
#define LAGCHECKTIME		70  /* Time between sending out lag NOTICE's */
#define LAGANNDELAY		30  /* The delay between lag announcements
				    ** for a specific server.  This prevents
				    ** LAGCHECK floods
				    */
#define HTMLDELAY		30  /* The delay before writing out the HTML
				    ** dump, used to make sure it's at least
				    ** part way sync'ed to it's uplink before
				    ** generating statistics
				    */
#define DUMPDELAY		30 /* The delay before writing out the data
				   ** dump
				   */
#define EXPIREPOLL		60 /* The time to wait between checks for
				   ** auto-expiration of splits
				   */
#define DEFAULT_DELAY		30 /* The number of seconds to delay sending
				   ** spy() messages
				   */

/* Hash table sizes must be prime */
#ifdef LARGE_NET
# define S_TABLE_SIZE 151
# define U_TABLE_SIZE 80387
#else
# define S_TABLE_SIZE	97
# define U_TABLE_SIZE	1999
#endif /* LARGE_NET */

#define T_TABLE_SIZE		97

#define TRUE 1
#define FALSE 0

/* Define the default time delay (in milliseconds) */
#ifdef DELAY_PRIVMSG
# define DELAYTIME 750000
#endif

/* If we disallow the PRIVMSG nick method, be sure and enable PRIVMSG u@h
** or else we cannot communicate with the service
*/
#ifdef DISALLOW_NICK_PRIVMSG
# define SUPPORT_MSG_UH
#endif

/* SplitServ defines */
#ifdef SPLITSERV
# define MAXSERVER 256
# define MAXLEAF 64
# define MAXDEPTH 32
#endif

/* JupeServ defines */
#ifdef JUPESERV
# define JUPEFILE "etc/jupes.cfg"
#endif

#endif
