/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** init.c - Stats initialization functions
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: init.c,v 1.10 2002/04/06 14:47:08 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

/* These are all horrible functions, peppered with ifdef's */

/* This is ugly, logging into each server type is slightly different */
void login()
{
#if defined(IRCD_HYBRID)
	sts("PASS %s :TS", settings.pass);
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
	sts("SVINFO 3 1 0 :%ld", time(NULL));
#else
#if defined(IRCD_HYB6)
	sts("PASS %s :TS", settings.pass);
	sts("CAPAB :QS");
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
	sts("SVINFO 5 1 0 :%ld", time(NULL));
#else
#if defined(IRCD_HYB7)
	/* NOTE:  a Stats service CANNOT be a LazyLink!!!!!!!!!
	** It's impossible for a Stats service to keep accurate
	** counts on anything if if doesn't receive a complete burst
	*/
	sts("PASS %s :TS", settings.pass);
	sts("CAPAB :QS HUB");
        sts("SERVER %s 1 :%s", settings.name, settings.desc);
        sts("SVINFO 5 3 0 :%ld", time(NULL));
#else
#if defined(IRCD_2_10)
	sts("PASS %s :021030003 IRC|%sHst P", settings.pass,
		IsDebug() ? "D" : "");
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
#endif
#endif
#endif
#endif
}

#ifdef JUPESERV

/* Bring up the jupe service */
void init_jupeserv()
{
        struct User *u = NULL;

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
        sts("NICK %s 1 1 +io %s %s %s :Jupe Service",
                JupeServ.nick, JupeServ.user, JupeServ.host, settings.name);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 %s %s %d +io :Jupe Service", JupeServ.nick,
               JupeServ.user, JupeServ.host, 1);
#endif
#endif

#ifdef IRCD_2_10
        AddUser(JupeServ.nick, JupeServ.user, JupeServ.host,
                findserver(settings.name));
#else
        AddUser(JupeServ.nick, JupeServ.user, JupeServ.host, settings.name);
#endif

        u = finduser(JupeServ.nick);
        IncreaseOpers(findserver(settings.name));
        u->flags |= UFLAG_ISOPER;
#ifdef OPER_MODES
        u->operflags |= OPER_GLOBAL;
#endif
}

#endif

#ifdef STATSERV
/* Init the StatServ client */
void init_statserv()
{

#ifdef SUPPORT_MSG_UH
	snprintf(StatServ.rname, LINE, "/msg %s HELP", s_UH);
#else
	snprintf(StatServ.rname, LINE, "/msg %s HELP", StatServ.nick);
#endif

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
	sts("NICK %s 1 1 %s %s %s %s :%s", StatServ.nick,
		IsSetOper() ? "+o" : "+", StatServ.user, StatServ.host,
		settings.name, StatServ.rname);
#else
#if defined(IRCD_2_10)
	sts("NICK %s 1 %s %s %d %s :%s", StatServ.nick,
		StatServ.user, StatServ.host, 
		/* This is the server token (which is ugly as sin) 
		** For now, I'll send '1'... Once I find out more about
		** it, I'll send the proper value.  
		*/
		1, IsSetOper() ? "+o" : "+", StatServ.rname);
#endif
#endif

#ifdef IRCD_2_10
	AddUser(StatServ.nick, StatServ.user, StatServ.host,
		findserver(settings.name));
#else
	AddUser(StatServ.nick, StatServ.user, StatServ.host, settings.name);
#endif

/* If the StatServ client is an oper, increase oper count here
** and set it as an oper
*/
	if (IsSetOper()) {
		struct User* u = finduser(StatServ.nick);
		IncreaseOpers(findserver(settings.name));
		u->flags |= UFLAG_ISOPER;
#ifdef OPER_MODES
		u->operflags |= OPER_GLOBAL;
#endif
	}
}
#endif /* STATSERV */

#ifdef SPAMSERV
/* Bring up the Spam detecting client if it was enabled */
void init_spam()
{

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
	sts("NICK %s 1 1 + please %s %s :Spam Me!", s_Spam,
		settings.name, settings.name);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 please %s %d + :Spam Me!", s_Spam,
                settings.name, 1);
                /* This is the server token myself is just 1
                */
#endif
#endif

#ifdef IRCD_2_10
	AddUser(s_Spam, "please", settings.name, findserver(settings.name));
#else
	AddUser(s_Spam, "please", settings.name, settings.name);
#endif
}
#endif

#ifdef SPLITSERV
/* Bring up the Split detecting/reporting client if it was enabled */
void init_splitserv()
{

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
        sts("NICK %s 1 1 + %s %s %s :%s", SplitServ.nick, SplitServ.user,
                SplitServ.host, settings.name, sp.desc);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 %s %s %d + :%s", SplitServ.nick, SplitServ.user,
                SplitServ.host, 1, sp.desc);
                /* This is the server token myself is 1
                */
#endif
#endif

#ifdef IRCD_2_10
        AddUser(SplitServ.nick, SplitServ.user, SplitServ.host,
		findserver(settings.name));
#else
        AddUser(SplitServ.nick, SplitServ.user, SplitServ.host, settings.name);
#endif
}
#endif

/* Bring up the administration service */
void init_admin()
{
	struct User *u = NULL;

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
        sts("NICK %s 1 1 +io %s %s %s :Administration Service",
		AdminServ.nick, AdminServ.user, AdminServ.host, settings.name);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 %s %s %d +io :Administration Service", AdminServ.nick, AdminServ.user,
                AdminServ.host, 1);
#endif
#endif

#ifdef IRCD_2_10
        AddUser(AdminServ.nick, AdminServ.user, AdminServ.host,
		findserver(settings.name));
#else
        AddUser(AdminServ.nick, AdminServ.user, AdminServ.host, settings.name);
#endif

	u = finduser(AdminServ.nick);
	IncreaseOpers(findserver(settings.name));
	u->flags |= UFLAG_ISOPER;
#ifdef OPER_MODES
        u->operflags |= OPER_GLOBAL;
#endif
}

#ifdef STATSERV
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Stats
*/
void init_join()
{
   dlink_node *dl = me.chan_list.head;

   while (dl)
   {
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
	sts(":%s SJOIN %ld %s + :@%s", settings.name, 
	    time(NULL), (char *) dl->data, StatServ.nick);
#else
#if defined(IRCD_2_10)
	/* Here is the only one that we hack ops... */
	sts(":%s NJOIN %s :@%s", settings.name, (char *) dl->data, 
		StatServ.nick);
#endif
#endif
	dl = dl->next;
   }
}
#endif

#ifdef SPLITSERV
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Split
*/
void init_spjoin()
{
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
	sts(":%s SJOIN %ld %s + :@%s", settings.name,
        	time(NULL), sp.join_chan, SplitServ.nick);
#else
#if defined(IRCD_2_10)
        /* Here is the only one that we hack ops... */
        sts(":%s NJOIN %s :@%s", settings.name, sp.join_chan, SplitServ.nick);
#endif
#endif
}
#endif

#ifdef ALERTMODE
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with AdminServ...
** Though in this case we might want to.....
*/
void init_alert_join()
{
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
        sts(":%s SJOIN %ld %s + :@%s", settings.name,
                time(NULL), ALERTCHAN, AdminServ.nick);
#else
#if defined(IRCD_2_10)
        /* Here is the only one that we hack ops... */
        sts(":%s NJOIN %s :@%s", settings.name, ALERTCHAN, AdminServ.nick);
#endif
#endif
}
#endif

#ifdef SPYMODE
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with AdminServ...
** Though in this case we might want to.....
*/
void init_spy_join()
{
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7)
        sts(":%s SJOIN %ld %s + :@%s", settings.name,
                time(NULL), SPYCHAN, AdminServ.nick);
#else
#if defined(IRCD_2_10)
        /* Here is the only one that we hack ops... */
        sts(":%s NJOIN %s :@%s", settings.name, SPYCHAN, AdminServ.nick);
#endif
#endif
}
#endif
