/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** split-data.c - Split service's backend functions
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: split-data.c,v 1.28 2002/04/06 20:15:00 wcampbel Exp $
*/

#include "config.h"
#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef SPLITSERV

void SpInitTables()
{
	int i;

        for (i=0; i<MAXSERVER; i++)
	{
                server_table[i] = NULL;
        }
        map_head = SpNewServer(settings.name, NULL);
	split_table.head = NULL;
	split_table.tail = NULL;
}

struct SpServer* SpNewServer(char* name, char* uplink)
{
	int i,j;
	struct SpServer* up = SpFindServer(uplink);
	struct SpServer* s = smalloc(sizeof(struct SpServer));
	strlcpy(s->name, name, MAXHOST);
	s->uplink = up;
	if (up)
	{
		up->numleaf++;
		for(j=0; j<MAXLEAF; j++)
		{
			if (!up->leaflist[j])
			{
				up->leaflist[j] = s;
				break;
			}
		}
	}
	else
	{
	    if (uplink != NULL) /* me... uplink will be NULL */
		log("SPLIT WARNING: Unable to find uplink: %s for server "
		    "name: %s", uplink, name);
	}
	for (i=0; i<MAXSERVER; i++)
	{
		if (!server_table[i])
		{
			server_table[i] = s;
			break;
		}
	}
	return s;
}

void SpDelServer(char* name)
{
	int i;
	int j;
	struct SpServer* s = NULL;
	if (name)
	{
		for (j=0; j<MAXSERVER; j++)
		{
			if (server_table[j])
			{
				if (!strcasecmp(name, server_table[j]->name))
				{
					s = server_table[j];
					server_table[j] = NULL;
				}
			}
		}
		if (s)
		{
		    if (s->uplink)
		    {
			s->uplink->numleaf--;

			for (i=0; i<MAXLEAF; i++)
			{
				if (s->uplink->leaflist[i])
				{
					if (!strcasecmp(name, 
					    s->uplink->leaflist[i]->name))
					{
						s->uplink->leaflist[i] = NULL;
					}
				}
			}

                    }
		    else
		    {
                        log("SPLIT: No uplink for server: %s", s->name);
		    }

		    free(s);
		}
		else
		{
		    log("SPLIT: No server table entry for server name: %s", name);
		}
	}
}

struct SpServer* SpFindServer(char* name)
{
	int i;
	if (name)
	{
		for (i=0; i<MAXSERVER; i++)
		{
			if (server_table[i])
			{
				if (!strcasecmp(server_table[i]->name, name))
				{
					return server_table[i];
				}
			}
		}
	}
	debug("SPLIT: Unable to FindServer for name: %s", name);
	return NULL;
}

void SpNewSplit(char * name)
{
	struct SpServer* s;
	struct SpServer *t = SpFindSplit(name);
	dlink_node *dl;
	if (t)
	{
		log("SPLIT: Split entry for server name: %s already exists",
			name);
		return;
	}
	s = smalloc(sizeof(struct SpServer));
	dl = smalloc(sizeof(dlink_node));
	strlcpy(s->name, name, MAXHOST);
	s->ts = time(NULL);
	/* None of the other variables are used here */
	dlinkAddTail(s, dl, &split_table);
}

void SpDelSplit(char * name)
{
	struct SpServer* s = NULL;
	dlink_node *dl = split_table.head;
	while (dl)
	{
		s = (struct SpServer *) dl->data;
		if (!strcasecmp(s->name, name))
		{
			dlinkDelete(dl, &split_table);
			free(s);
			free(dl);
			return;
		}
		/* This is safe, won't cause Linux cores */
		dl = dl->next;
	}
#if 0
	/* New servers will get logged by this, too much useless
	** clutter in the log file
	*/
	if (!is_delayed)
		log("SPLIT: Unable to DelSplit server name: %s", name);
#endif
}

struct SpServer* SpFindSplit(char* name)
{
	dlink_node *dl = NULL;
	struct SpServer *spl = NULL;
        if (name)
	{
		for (dl = split_table.head; dl; dl = dl->next)
		{
			spl = (struct SpServer *) dl->data;
			if (!strcasecmp(spl->name, name))
			{
				return spl;
			}
		}
        }
	debug("SPLIT: Unable to FindSplit server name: %s", name);
        return NULL;
}

#endif
