/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** split-util.c - Split service's utility functions
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: split-util.c,v 1.46 2002/04/02 21:05:25 wcampbel Exp $
*/

#include "config.h"
#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef SPLITSERV

static char* dash(int);
static char* hdash(int);
static void dump_text(FILE*, struct SpServer*, int);

void dump_map()
{
	FILE *fp = fopen(sp.text_name, "w");
	dump_text(fp, map_head, 0);
	fclose(fp);
}
	
static void dump_text(FILE *fp, struct SpServer* s, int level)
{
        int i;
        char* d = dash(level);
        if (s)
	{
                fprintf(fp, "%s %s\n", d, s->name);
                for (i=0; i<MAXLEAF; i++)
		{
                        if (s->leaflist[i])
			{
                                if (s->leaflist[i]->name)
				{
                                        dump_text(fp, s->leaflist[i], level+1);
                                }
                        }
                }
        }
}

void html_map(FILE *fp, struct SpServer* s, int level)
{
        int i;
        char* d = hdash(level);
        if (s)
	{
                fprintf(fp, "%s <A HREF=\"#%s\">%s</A><BR>\n", d,
			s->name, s->name);
                for (i=0; i<MAXLEAF; i++)
		{
                        if (s->leaflist[i])
			{
                                if (s->leaflist[i]->name)
				{
                                        html_map(fp, s->leaflist[i], level+1);
                                }
                        }
                }
        }
}

/* REWRITE THIS DAMN FUNCTION */
static char* dash(int level)
{
        int i;
        static char s[MAXDEPTH * 4];
        s[0] = '\0';
        if (!level)
	{
                return s;
        }
        s[0] = ' ';
        s[1] = '\0';
        for (i=level - 1; i>=0; i--)
	{
                if (!i)
		{
			/* All these strlcat's are hacking around glibc
			** brokenness
			*/
			strlcat(s, "`-", MAXDEPTH * 4);
                        /* snprintf(s, MAXDEPTH * 4, "%s`-", s); */
                }
		else
		{
			strlcat(s, "   ", MAXDEPTH * 4);
                        /* snprintf(s, MAXDEPTH * 4, "%s   ", s); */
                }
        }
        return s;
}

static char* hdash(int level)
{
        int i;
        static char s[MAXDEPTH * 10];
        s[0] = '\0';
        if (!level)
	{
                return s;
        }
	s[0] = '&';
        s[1] = 'n';
	s[2] = 'b';
	s[3] = 's';
	s[4] = 'p';
	s[5] = ';';
        s[6] = '\0';
        for (i=level - 1; i>=0; i--)
	{
                if (!i)
		{
			/* More hacks because of glibc */
			strlcat(s, "`-", MAXDEPTH*10);
                        /* snprintf(s, MAXDEPTH*10, "%s`-", s); */
                }
		else
		{
			strlcat(s, "&nbsp;&nbsp;&nbsp;", MAXDEPTH*10);
                        /* snprintf(s, MAXDEPTH*10,
			**          "%s&nbsp;&nbsp;&nbsp;", s);
			*/
                }
        }
        return s;
}

void SpFlushServerTree(struct SpServer *s)
{
        int i;
        if (s)
	{
                for (i=0; i<MAXLEAF; i++)
		{
                        if(s->leaflist[i])
			{
                                SpFlushServerTree(s->leaflist[i]);
                                s->leaflist[i]=NULL;
                        }
                }
                /* Damn, extra calls to findserver :( */
                SpDelServer(s->name);
                /* Don't add things to the split table */
        }
}

void PrintMap(char* target, struct SpServer* s, int level)
{
        int i;
        struct timeval tv;
        char* d = dash(level);
        if (s)
	{
                privmsg(target, SplitServ.nick, MSG_SPDEFAULT, "%s %s", d,
			s->name);
                tv.tv_usec = 750000; /* 3/4 a second delay */
                tv.tv_sec = 0;       /* ignore this */
                select(1, NULL, NULL, NULL, &tv);
                for (i=0; i<MAXLEAF; i++)
		{
                        if (s->leaflist[i])
			{
                                if (s->leaflist[i]->name)
				{
                                        PrintMap(target, s->leaflist[i],
                                                (level+1));
                                }
                        }
                }
        }
}

void PrintSplit(char* target)
{
        int count = 0;
        char * td;
        struct timeval tv;
        time_t now = time(NULL);
	dlink_node *dl = NULL;
	struct SpServer *spl = NULL;

        /* Everything in the Split List will have been created with
           NewSplit, so it will have a valid time stamp */

	for (dl = split_table.head; dl; dl = dl->next)
	{
		spl = (struct SpServer *) dl->data;
		td = timediff(now - spl->ts);
		count++;
		privmsg(target, SplitServ.nick, MSG_SPDEFAULT,
			"SPLIT SERVER: %s for %s", spl->name, td);
		tv.tv_usec = 750000; /* 3/4 a second delay */
		tv.tv_sec = 0;       /* ignore this */
		select(1, NULL, NULL, NULL, &tv);
	}
        if (count)
	{
		privmsg(target, SplitServ.nick, MSG_SPDEFAULT,
			"%d TOTAL SPLITS", count);
	}
	else
	{
                privmsg(target, SplitServ.nick, MSG_SPDEFAULT, "NO SPLITS");
        }
}

void PrintDump(char * target)
{
        int i,j;
        char leaves[BUFSIZE] = "";
        struct timeval tv;
        for (i=0; i<MAXSERVER; i++)
	{
                leaves[0] = '\0';
                if (server_table[i])
		{
                        for (j=0; j<MAXLEAF; j++)
			{
                                if (server_table[i]->leaflist[j])
				{
					/* ugh hacks - fix glibc!
                                        ** snprintf(leaves, BUFSIZE, "%s %s",
					**   leaves,
                                        **  server_table[i]->leaflist[j]->name);
					*/
					strlcat(leaves, " ", BUFSIZE);
					strlcat(leaves,
					  server_table[i]->leaflist[j]->name,
					  BUFSIZE);
                                }
                        }
                        privmsg(target, SplitServ.nick, MSG_SPDEFAULT,
				"Server: %s Leaves:%s", server_table[i]->name,
                                (!strcmp(leaves, "")) ?  " None" : leaves);
                        tv.tv_usec = 750000; /* 3/4 a second delay */
                        tv.tv_sec = 0;       /* ignore this */
                        select(1, NULL, NULL, NULL, &tv);
                }
        }
}

void Update(char *origin)
{
        if (IsSPEnableText())
        {
                dump_map();
                privmsg(origin, SplitServ.nick, MSG_SPDEFAULT,
			"MAP file updated");
            return;
        }
        privmsg(origin, SplitServ.nick, MSG_SPDEFAULT,
		"MAP Text Dump Disabled");
        return;
}

void Add(char * origin, char * serv)
{
        struct SpServer *s = NULL;

        s = SpFindServer(serv);
        if (s)
	{
           privmsg(origin, SplitServ.nick, MSG_SPDEFAULT,
		"Can't add a split for an existing server");
           return;
        }
        s = SpFindSplit(serv);
        if (s)
	{
           privmsg(origin, SplitServ.nick, MSG_SPDEFAULT,
		"Can't add a split for an existing split");
           return;
        }
        SpNewSplit(serv);
        privmsg(sp.join_chan, SplitServ.nick, MSG_SPDEFAULT,
		"ADDED SPLIT: %s", serv);
        splog("Added split %s by %s", serv, origin);
        return;
}

void Forget(char * origin, char * serv)
{
        struct SpServer * s = NULL;
	dlink_node *dl = NULL;

	for (dl = split_table.head; dl; dl = dl->next)
	{
		s = (struct SpServer *) dl->data;
		if (!strcasecmp(s->name, serv))
		{
			privmsg(origin, SplitServ.nick, MSG_SPDEFAULT,
				"Removed from the Split Table", s->name);
			dlinkDelete(dl, &split_table);
			privmsg(sp.join_chan, SplitServ.nick, MSG_SPDEFAULT,
				"FORGOT SPLIT: %s", s->name);
			splog("Forgot split %s by %s", s->name, origin);
			free(s);
			free(dl);
			return;
		}
	}

        privmsg(origin, SplitServ.nick, MSG_SPDEFAULT,
		"%s NOT found in the Split Table", serv);
        return;
}

void expire_splits(void)
{
        struct SpServer * s = NULL;
	dlink_node *dl = split_table.head;
	dlink_node *tdl = NULL;
	time_t now = time(NULL);

	while (dl)
	{
		s = (struct SpServer *) dl->data;
		if ((now - s->ts) > sp.expire_time)
		{
			tdl = dl->next;
			dlinkDelete(dl, &split_table);
			privmsg(sp.join_chan, SplitServ.nick, MSG_SPDEFAULT,
				"SPLIT EXPIRED AFTER %s: %s",
				timediff(now - s->ts), s->name);
			splog("Split %s expired after %s", s->name,
				timediff(now - s->ts));
			free(s);
			free(dl);
			/* Can't use freed memory...expecially on Linux...
			** one thing they did do right...
			*/
			dl = tdl;
		}
		else
		{
			dl = dl->next;
		}
	}
}


#endif
